package com.fr.plugin.sqy.surface.chart;

import com.fanruan.api.json.JSONKit;
import com.fr.config.predefined.ColorFillStyle;
import com.fr.extended.chart.AbstractChart;
import com.fr.extended.chart.HyperLinkPara;
import com.fr.extended.chart.StringFormula;
import com.fr.js.JavaScript;
import com.fr.js.NameJavaScript;
import com.fr.js.NameJavaScriptGroup;
import com.fr.json.*;
import com.fr.plugin.sqy.surface.PluginConstants;
import com.fr.plugin.sqy.surface.config.AxisConfig;
import com.fr.plugin.sqy.surface.config.LineStyleConfig;
import com.fr.plugin.sqy.surface.option.ChartOption;
import com.fr.plugin.sqy.surface.option.SurfaceChartOptionBuilder;
import com.fr.plugin.sqy.surface.option.modle.ChartOptionDirector;
import com.fr.stable.AssistUtils;
import com.fr.stable.ParameterProvider;
import com.fr.stable.web.Repository;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;

import java.util.ArrayList;
import java.util.List;

public class Chart extends AbstractChart<ChartDataConfig> {
    private static Chart chart;
    private ChartType chartType = ChartType.TYPE1;
    private ColorFillStyle colorFillStyle = new ColorFillStyle();
    private StringFormula visualMin = new StringFormula();
    private StringFormula visualMax = new StringFormula();
    private LineStyleConfig wireFrameConfig = new LineStyleConfig("wireframe");
    private AxisConfig axisConfigX = new AxisConfig("xAxis3d");
    private AxisConfig axisConfigY = new AxisConfig("yAxis3d");
    private AxisConfig axisConfigZ = new AxisConfig("zAxis3d");


    public Chart() {
    }

    public ColorFillStyle getColorFillStyle() {
        return colorFillStyle;
    }

    public void setColorFillStyle(ColorFillStyle colorFillStyle) {
        this.colorFillStyle = colorFillStyle;
    }

    public StringFormula getVisualMin() {
        return visualMin;
    }

    public void setVisualMin(StringFormula visualMin) {
        this.visualMin = visualMin;
    }

    public StringFormula getVisualMax() {
        return visualMax;
    }

    public void setVisualMax(StringFormula visualMax) {
        this.visualMax = visualMax;
    }

    public LineStyleConfig getWireFrameConfig() {
        return wireFrameConfig;
    }

    public void setWireFrameConfig(LineStyleConfig wireFrameConfig) {
        this.wireFrameConfig = wireFrameConfig;
    }

    public AxisConfig getAxisConfigX() {
        return axisConfigX;
    }

    public void setAxisConfigX(AxisConfig axisConfigX) {
        this.axisConfigX = axisConfigX;
    }

    public AxisConfig getAxisConfigY() {
        return axisConfigY;
    }

    public void setAxisConfigY(AxisConfig axisConfigY) {
        this.axisConfigY = axisConfigY;
    }

    public AxisConfig getAxisConfigZ() {
        return axisConfigZ;
    }

    public void setAxisConfigZ(AxisConfig axisConfigZ) {
        this.axisConfigZ = axisConfigZ;
    }

    @Override
    protected void readAttr(XMLableReader reader) {
        super.readAttr(reader);
        //用来标记主属性
        String attrAsString = reader.getAttrAsString("mainAttr", "");
        if("main".equals(attrAsString)){
            this.setChartType(ChartType.parseInt(reader.getAttrAsInt("chartType", 0)));
            this.getVisualMin().readAttr("visualMin",reader);
            this.getVisualMax().readAttr("visualMax",reader);
            this.getWireFrameConfig().readAttr("",reader);
            this.getAxisConfigX().readAttr("",reader);
            this.getAxisConfigY().readAttr("",reader);
            this.getAxisConfigZ().readAttr("",reader);
        }
    }

    @Override
    public void readXML(XMLableReader xmLableReader) {
        super.readXML(xmLableReader);
        colorFillStyle.readXML(xmLableReader);
    }

    @Override
    protected void writeAttr(XMLPrintWriter writer) {
        super.writeAttr(writer);
        writer.attr("mainAttr", "main");
        writer.attr("chartType", getChartType().ordinal());
        this.getVisualMax().writeAttr("visualMax",writer);
        this.getVisualMin().writeAttr("visualMin",writer);
        this.getWireFrameConfig().writeAttr("",writer);
        this.getAxisConfigX().writeAttr("",writer);
        this.getAxisConfigY().writeAttr("",writer);
        this.getAxisConfigZ().writeAttr("",writer);
    }

    @Override
    public void writeXML(XMLPrintWriter xmlPrintWriter) {
        super.writeXML(xmlPrintWriter);
        colorFillStyle.writeXML(xmlPrintWriter);
    }

    @Override
    protected void addJSON(ChartDataConfig surfaceChartDataConfig, JSONObject jsonObject, Repository repository, AbstractChart<ChartDataConfig>.JSONPara jsonPara) throws JSONException {
        //去掉试用字符
        jsonObject.remove("trialLicenseWater");
        jsonObject.put("trialLicenseWater","");

        NameJavaScriptGroup linkGroup = this.getLinkGroup();
        JSONArray jsArray = JSONKit.createJSONArray();
        if(linkGroup != null && linkGroup.size() >= 1){
            for (int i = 0; i < linkGroup.size(); i++) {
                jsArray.put(linkGroup.getNameHyperlink(i).getJavaScript().createJSONObject(repository));
            }
        }
        jsonObject.put("links",jsArray);


        SurfaceChartOptionBuilder surfaceChartOptionBuilder = new SurfaceChartOptionBuilder();
        if (surfaceChartDataConfig != null) {
            surfaceChartOptionBuilder.setValuesX(surfaceChartDataConfig.getX().getValues());
            surfaceChartOptionBuilder.setValuesY(surfaceChartDataConfig.getY().getValues());
            surfaceChartOptionBuilder.setSeriesFiles(surfaceChartDataConfig.getCustomFields());
            surfaceChartOptionBuilder.setCustomValueField(surfaceChartDataConfig.getCustomValueField().getValues());
        }
        surfaceChartOptionBuilder.setColors(this.getColorFillStyle().getColorList());
        surfaceChartOptionBuilder.setVisualMax(this.getVisualMax().getResult());
        surfaceChartOptionBuilder.setVisualMin(this.getVisualMin().getResult());
        surfaceChartOptionBuilder.setWireFrameConfig(this.getWireFrameConfig());
        surfaceChartOptionBuilder.setAxisConfigX(this.getAxisConfigX());
        surfaceChartOptionBuilder.setAxisConfigY(this.getAxisConfigY());
        surfaceChartOptionBuilder.setAxisConfigZ(this.getAxisConfigZ());
        //构建echarts的option对象
        ChartOption chartOptionByModel = ChartOptionDirector.createChartOptionByModel(surfaceChartOptionBuilder);
        jsonObject.put("data",chartOptionByModel.getJSONObject());
    }



    @Override
    public int hashCode() {
        return super.hashCode() + AssistUtils.hashCode(getChartType(),getVisualMin(),getVisualMax());
    }

    @Override
    public boolean equals(Object ob) {
        return ob instanceof Chart
                && super.equals(ob)
                && AssistUtils.equals(this.getChartType(), ((Chart) ob).getChartType())
                && AssistUtils.equals(this.getVisualMin(), ((Chart) ob).getVisualMin())
                && AssistUtils.equals(this.getVisualMax(), ((Chart) ob).getVisualMax());
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        Chart clone = (Chart)super.clone();
        clone.setChartType(this.getChartType());
        clone.setVisualMax(this.getVisualMax().clone());
        clone.setVisualMin(this.getVisualMin().clone());
        return clone;
    }

    @Override
    protected List<StringFormula> formulas() {
        ArrayList<StringFormula> stringFormulas = new ArrayList<>();
        stringFormulas.add(this.getVisualMin());
        stringFormulas.add(this.getVisualMax());
        stringFormulas.add(this.getAxisConfigX().getName());
        stringFormulas.add(this.getAxisConfigX().getMax());
        stringFormulas.add(this.getAxisConfigX().getMin());
        stringFormulas.add(this.getAxisConfigY().getName());
        stringFormulas.add(this.getAxisConfigY().getMax());
        stringFormulas.add(this.getAxisConfigY().getMin());
        stringFormulas.add(this.getAxisConfigZ().getName());
        stringFormulas.add(this.getAxisConfigZ().getMax());
        stringFormulas.add(this.getAxisConfigZ().getMin());

        return stringFormulas;
    }


    @Override
    protected String[] requiredJS() {
        return new String[]{
                "com/fr/plugin/sqy/surface/js/index.js",
                "com/fr/plugin/sqy/surface/js/echarts.js",
                "com/fr/plugin/sqy/surface/js/echarts-gl.js"
        };
    }

    @Override
    protected String[] requiredCSS() {
        return new String[]{
                "com/fr/plugin/sqy/surface/css/index.css"
        };
    }

    @Override
    protected String getChartID() {
        return PluginConstants.PLOT_ID;
    }

    @Override
    public String getChartName() {
        return PluginConstants.CHART_NAME;
    }

    @Override
    protected String wrapperName() {
        return "surfaceWrapper";
    }

    @Override
    protected HyperLinkPara[] hyperLinkParas() {
        return new HyperLinkPara[]{
                new HyperLinkPara() {
                    @Override
                    public String getName() {
                        return "X值";
                    }

                    @Override
                    public String getFormulaContent() {
                        return "X";
                    }

                    @Override
                    public String[] getProps() {
                        return new String[]{"data", "x"};
                    }
                },
                new HyperLinkPara() {
                    @Override
                    public String getName() {
                        return "Y值";
                    }

                    @Override
                    public String getFormulaContent() {
                        return "Y";
                    }

                    @Override
                    public String[] getProps() {
                        return new String[]{"data", "y"};
                    }
                },
                new HyperLinkPara() {
                    @Override
                    public String getName() {
                        return "Z值";
                    }

                    @Override
                    public String getFormulaContent() {
                        return "Z";
                    }

                    @Override
                    public String[] getProps() {
                        return new String[]{"data", "z"};
                    }
                },
                new HyperLinkPara() {
                    @Override
                    public String getName() {
                        return "系列名";
                    }

                    @Override
                    public String getFormulaContent() {
                        return "SERIES_NAME";
                    }

                    @Override
                    public String[] getProps() {
                        return new String[]{"data", "SERIES_NAME"};
                    }
                }
        };
    }

    @Override
    protected String demoImagePath() {
        return "com/fr/plugin/sqy/surface/img/face.png";
    }

    public ChartType getChartType() {
        return chartType;
    }

    public void setChartType(ChartType chartType) {
        this.chartType = chartType;
    }
}
