package com.fr.plugin.sqy.surface.option;

import com.fanruan.api.json.JSONKit;
import com.fr.extended.chart.ExtendedField;
import com.fr.general.GeneralUtils;
import com.fr.json.JSONArray;
import com.fr.plugin.sqy.surface.config.AxisConfig;
import com.fr.plugin.sqy.surface.config.LineStyleConfig;
import com.fr.plugin.sqy.surface.option.modle.ChartOptionProvide;
import com.fr.plugin.sqy.surface.zenum.AxisTypeEnum;
import com.fr.plugin.sqy.surface.util.PluginStringUtils;
import com.fr.plugin.sqy.surface.zenum.LineTypeEnum;
import com.fr.stable.StringUtils;

import java.awt.*;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

public class SurfaceChartOptionBuilder implements ChartOptionProvide {
    private ChartOption chartOption;
    private List<Object> valuesX;
    private List<Object> valuesY;
    private List<Object> customValueField;
    private List<ExtendedField> seriesFiles;
    private List<Color> colors;
    private String visualMin;
    private String visualMax;
    private LineStyleConfig wireFrameConfig;
    private AxisConfig axisConfigX;
    private AxisConfig axisConfigY;
    private AxisConfig axisConfigZ;

    public AxisConfig getAxisConfigX() {
        return axisConfigX;
    }

    public void setAxisConfigX(AxisConfig axisConfigX) {
        this.axisConfigX = axisConfigX;
    }

    public AxisConfig getAxisConfigY() {
        return axisConfigY;
    }

    public void setAxisConfigY(AxisConfig axisConfigY) {
        this.axisConfigY = axisConfigY;
    }

    public AxisConfig getAxisConfigZ() {
        return axisConfigZ;
    }

    public void setAxisConfigZ(AxisConfig axisConfigZ) {
        this.axisConfigZ = axisConfigZ;
    }

    public LineStyleConfig getWireFrameConfig() {
        return wireFrameConfig;
    }

    public void setWireFrameConfig(LineStyleConfig wireFrameConfig) {
        this.wireFrameConfig = wireFrameConfig;
    }

    public String getVisualMin() {
        return visualMin;
    }

    public void setVisualMin(String visualMin) {
        this.visualMin = visualMin;
    }

    public String getVisualMax() {
        return visualMax;
    }

    public void setVisualMax(String visualMax) {
        this.visualMax = visualMax;
    }

    public List<Color> getColors() {
        return colors;
    }

    public void setColors(List<Color> colors) {
        this.colors = colors;
    }

    public List<Object> getCustomValueField() {
        return customValueField;
    }

    public void setCustomValueField(List<Object> customValueField) {
        this.customValueField = customValueField;
    }

    public List<Object> getValuesX() {
        return valuesX;
    }

    public void setValuesX(List<Object> valuesX) {
        this.valuesX = valuesX;
    }

    public List<Object> getValuesY() {
        return valuesY;
    }

    public void setValuesY(List<Object> valuesY) {
        this.valuesY = valuesY;
    }

    public List<ExtendedField> getSeriesFiles() {
        return seriesFiles;
    }

    public void setSeriesFiles(List<ExtendedField> seriesFiles) {
        this.seriesFiles = seriesFiles;
    }

    public SurfaceChartOptionBuilder() {
        chartOption = new ChartOption();
    }

    @Override
    public void buildToolTip() {
    }

    @Override
    public void buildVisualMap() {
        VisualMap visualMap = new VisualMap();
        if(this.getColors() != null){
            VisualMap.InRange inRange = new VisualMap.InRange();
            inRange.setColor(this.getColors());
            visualMap.setInRange(inRange);
        }
        double min = Double.MAX_VALUE;
        double max = Double.MIN_VALUE;
        if(this.getCustomValueField() != null) {
            for (int i = 0; i < this.getCustomValueField().size(); i++) {
                double v = GeneralUtils.objectToNumber(this.getCustomValueField().get(i)).doubleValue();
                min = Math.min(v, min);
                max = Math.max(v, max);
            }
        }
        if(StringUtils.isNotBlank(visualMin) && PluginStringUtils.isNumber(visualMin)){
            min = Double.parseDouble(visualMin);
        }
        if(StringUtils.isNotBlank(visualMax) && PluginStringUtils.isNumber(visualMax)){
            max = Double.parseDouble(visualMax);
        }
        visualMap.setMax(max);
        visualMap.setMin(min);
        this.chartOption.setVisualMap(visualMap);
    }

    @Override
    public void buildAxis3Dx() {
        Axis3D axis3D = new Axis3D();
        AxisConfig axisConfig = this.getAxisConfigX();
        //设置轴标题
        if(axisConfig.getNameTextStyle().isShow()){
            axis3D.setName(axisConfig.getName().getResult());
            axis3D.setNameTextStyle(new LabelStyle(axisConfig.getNameTextStyle().getColor(), axisConfig.getNameTextStyle().getFontSize()));
        }else{
            axis3D.setName("");
        }
        //设置轴标签
        if(axisConfig.getAxisLabel().isShow()){
            axis3D.setAxisLabel(new LabelStyle(axisConfig.getAxisLabel().getColor(),axisConfig.getAxisLabel().getFontSize()));
            if(StringUtils.isNotBlank(axisConfig.getAxisLabel().getFormatter())){
                axis3D.getAxisLabel().setFormatter(axisConfig.getAxisLabel().getFormatter());
            }
        }
        //设置轴线
        if(LineTypeEnum.parseString(axisConfig.getAxisLine().getType()).ordinal() == 0){
            axis3D.setAxisLine(new AxisLine(false));
        }else{
            axis3D.setAxisLine(new AxisLine(true,new LineStyle(axisConfig.getAxisLine().getColor(),axisConfig.getAxisLine().getWidth(),axisConfig.getAxisLine().getType())));
            axis3D.setAxisTick(new AxisTick(axisConfig.getAxisTick().isShow()));
        }
        //设置网格线
        if(LineTypeEnum.parseString(axisConfig.getSplitLine().getType()).ordinal() == 0){
            axis3D.setSplitLine(new SplitLine(false));
        }else{
            axis3D.setSplitLine(new SplitLine(true,new LineStyle(axisConfig.getSplitLine().getColor(),axisConfig.getSplitLine().getWidth(),axisConfig.getSplitLine().getType())));
        }

        //设置值类型
        if(AxisTypeEnum.parseString(axisConfig.getType()).ordinal() == 0){
            if(this.getValuesX()!=null && this.getValuesX().size()>0 && (this.getValuesX().get(0) instanceof String)){
                axis3D.setType(AxisTypeEnum.CATEGORY.getName());
                axis3D.setBoundaryGap(false);
                axis3D.setMax(null);
                axis3D.setMin(null);
            }else{
                axis3D.setType(axisConfig.getType());
                String min = axisConfig.getMin().getResult();
                if(PluginStringUtils.isNumber(min)){
                    axis3D.setMin(Double.valueOf(min));
                }
                String max = axisConfig.getMax().getResult();
                if(PluginStringUtils.isNumber(max)){
                    axis3D.setMax(Double.valueOf(max));
                }
            }
        }else{
            axis3D.setType(axisConfig.getType());
            axis3D.setBoundaryGap(false);
        }

        this.chartOption.setAxis3Dx(axis3D);
    }

    @Override
    public void buildAxis3Dy() {
        Axis3D axis3D = new Axis3D();
        AxisConfig axisConfig = this.getAxisConfigY();
        //设置轴标题
        if(axisConfig.getNameTextStyle().isShow()){
            axis3D.setName(axisConfig.getName().getResult());
            axis3D.setNameTextStyle(new LabelStyle(axisConfig.getNameTextStyle().getColor(), axisConfig.getNameTextStyle().getFontSize()));
        }else{
            axis3D.setName("");
        }
        //设置轴标签
        if(axisConfig.getAxisLabel().isShow()){
            axis3D.setAxisLabel(new LabelStyle(axisConfig.getAxisLabel().getColor(),axisConfig.getAxisLabel().getFontSize()));
            if(StringUtils.isNotBlank(axisConfig.getAxisLabel().getFormatter())){
                axis3D.getAxisLabel().setFormatter(axisConfig.getAxisLabel().getFormatter());
            }
        }
        //设置轴线
        if(LineTypeEnum.parseString(axisConfig.getAxisLine().getType()).ordinal() == 0){
            axis3D.setAxisLine(new AxisLine(false));
        }else{
            axis3D.setAxisLine(new AxisLine(true,new LineStyle(axisConfig.getAxisLine().getColor(),axisConfig.getAxisLine().getWidth(),axisConfig.getAxisLine().getType())));
            axis3D.setAxisTick(new AxisTick(axisConfig.getAxisTick().isShow()));
        }
        //设置网格线
        if(LineTypeEnum.parseString(axisConfig.getSplitLine().getType()).ordinal() == 0){
            axis3D.setSplitLine(new SplitLine(false));
        }else{
            axis3D.setSplitLine(new SplitLine(true,new LineStyle(axisConfig.getSplitLine().getColor(),axisConfig.getSplitLine().getWidth(),axisConfig.getSplitLine().getType())));
        }

        //设置值类型
        if(AxisTypeEnum.parseString(axisConfig.getType()).ordinal() == 0){
            if(this.getValuesY()!=null && this.getValuesY().size()>0 && (this.getValuesY().get(0) instanceof String)){
                axis3D.setType(AxisTypeEnum.CATEGORY.getName());
                axis3D.setBoundaryGap(false);
                axis3D.setMax(null);
                axis3D.setMin(null);
            }else{
                axis3D.setType(axisConfig.getType());
                String min = axisConfig.getMin().getResult();
                if(PluginStringUtils.isNumber(min)){
                    axis3D.setMin(Double.valueOf(min));
                }
                String max = axisConfig.getMax().getResult();
                if(PluginStringUtils.isNumber(max)){
                    axis3D.setMax(Double.valueOf(max));
                }
            }
        }else{
            axis3D.setType(axisConfig.getType());
            axis3D.setBoundaryGap(false);
        }

        this.chartOption.setAxis3Dy(axis3D);
    }

    @Override
    public void buildAxis3Dz() {
        Axis3D axis3D = new Axis3D();
        AxisConfig axisConfig = this.getAxisConfigZ();
        //设置轴标题
        if(axisConfig.getNameTextStyle().isShow()){
            axis3D.setName(axisConfig.getName().getResult());
            axis3D.setNameTextStyle(new LabelStyle(axisConfig.getNameTextStyle().getColor(), axisConfig.getNameTextStyle().getFontSize()));
        }else{
            axis3D.setName("");
        }
        //设置轴标签
        if(axisConfig.getAxisLabel().isShow()){
            axis3D.setAxisLabel(new LabelStyle(axisConfig.getAxisLabel().getColor(),axisConfig.getAxisLabel().getFontSize()));
            if(StringUtils.isNotBlank(axisConfig.getAxisLabel().getFormatter())){
                axis3D.getAxisLabel().setFormatter(axisConfig.getAxisLabel().getFormatter());
            }
        }
        //设置轴线
        if(LineTypeEnum.parseString(axisConfig.getAxisLine().getType()).ordinal() == 0){
            axis3D.setAxisLine(new AxisLine(false));
        }else{
            axis3D.setAxisLine(new AxisLine(true,new LineStyle(axisConfig.getAxisLine().getColor(),axisConfig.getAxisLine().getWidth(),axisConfig.getAxisLine().getType())));
            axis3D.setAxisTick(new AxisTick(axisConfig.getAxisTick().isShow()));
        }
        //设置网格线
        if(LineTypeEnum.parseString(axisConfig.getSplitLine().getType()).ordinal() == 0){
            axis3D.setSplitLine(new SplitLine(false));
        }else{
            axis3D.setSplitLine(new SplitLine(true,new LineStyle(axisConfig.getSplitLine().getColor(),axisConfig.getSplitLine().getWidth(),axisConfig.getSplitLine().getType())));
        }

        //设置值类型
        if(AxisTypeEnum.parseString(axisConfig.getType()).ordinal() == 0){
            if(this.getCustomValueField()!=null && this.getCustomValueField().size()>0 && (this.getCustomValueField().get(0) instanceof String)){
                axis3D.setType(AxisTypeEnum.CATEGORY.getName());
                axis3D.setBoundaryGap(false);
                axis3D.setMax(null);
                axis3D.setMin(null);
            }else{
                axis3D.setType(axisConfig.getType());
                String min = axisConfig.getMin().getResult();
                if(PluginStringUtils.isNumber(min)){
                    axis3D.setMin(Double.valueOf(min));
                }
                String max = axisConfig.getMax().getResult();
                if(PluginStringUtils.isNumber(max)){
                    axis3D.setMax(Double.valueOf(max));
                }
            }
        }else{
            axis3D.setType(axisConfig.getType());
            axis3D.setBoundaryGap(false);
        }
        this.chartOption.setAxis3Dz(axis3D);
    }


    @Override
    public void buildGrid3D() {
        this.chartOption.setGrid3D(new Grid3D());
    }

    @Override
    public void buildSeries() {
        List<Series> seriesList = new ArrayList<>();
        if(this.getSeriesFiles() != null) {
            Wireframe wireframe = null;
            if(this.getWireFrameConfig() != null){
                wireframe = new Wireframe();
                boolean isShow = LineTypeEnum.parseString(this.getWireFrameConfig().getType()).ordinal() != 0;
                wireframe.setShow(isShow);
                if(isShow){
                    LineStyle lineStyle = new LineStyle();
                    lineStyle.setColor(this.getWireFrameConfig().getColor());
                    lineStyle.setWidth(this.getWireFrameConfig().getWidth());
                    lineStyle.setType(this.getWireFrameConfig().getType());
                    wireframe.setLineStyle(lineStyle);
                }
            }
            for (ExtendedField seriesFile : this.getSeriesFiles()) {
                seriesList.add(this.buildSeries(seriesFile,wireframe));
            }
        }

        this.chartOption.setSeries(seriesList);
    }

    private Series buildSeries(ExtendedField values,Wireframe wireframe){
        Series series = new Series();
        series.setWireframe(wireframe);
        List<Object> customValues = values.getValues();
        series.setName(values.getCustomName());
        if(this.getValuesX() != null && this.getValuesX().size() > 0){
            List<Object> dataList = new ArrayList<>();
            for (int i = 0; i < valuesY.size(); i++) {
                dataList.add(JSONKit.createJSONArray()
                        .put(valuesX.size()>i ? valuesX.get(i) : 0)
                        .put(valuesY.get(i))
                        .put(customValues.size()>i ? customValues.get(i) : 0)
                );
            }
            //需要对数据进行排序，否则图形出不来
            dataList.sort(new Comparator<Object>() {
                public int compare(Object o1,Object o2){
                    try {
                        Object o1col = ((JSONArray)o1).get(1);
                        Object o2col = ((JSONArray)o2).get(1);
                        int i;
                        if(o1col instanceof Number){
                            i = Double.valueOf(String.valueOf(o1col)).compareTo(Double.valueOf(String.valueOf(o2col)));
                        }else{
                            i = String.valueOf(o1col).compareTo(String.valueOf(o2col));
                        }
                        if(i != 0){
                            return i;
                        }
                        o1col = ((JSONArray)o1).get(0);
                        o2col = ((JSONArray)o2).get(0);
                        if(o1col instanceof Number){
                            return Double.valueOf(String.valueOf(o1col)).compareTo(Double.valueOf(String.valueOf(o2col)));
                        }else{
                            return String.valueOf(o1col).compareTo(String.valueOf(o2col));
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        return 0;
                    }
                }
            });
            series.setData(dataList);
        }
        return series;
    }



    @Override
    public ChartOption createChartOption() {
        return chartOption;
    }
}
