package com.fr.plugin.sqy.surface.option;


import com.fanruan.api.json.JSONKit;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.sqy.surface.option.modle.OptionProvider;
import com.fr.plugin.sqy.surface.util.PluginStringUtils;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 视觉隐射组件配置
 */
public class VisualMap implements OptionProvider {

    /**
     * 类型，默认连续型
     */
    private String type = "continuous";
    /**
     * 指定 visualMapContinuous 组件的允许的最小值。'min' 必须用户指定。[visualMap.min, visualMax.max] 形成了视觉映射的『定义域』。
     */
    private double min = -1D;
    /**
     * 指定 visualMapContinuous 组件的允许的最大值。'max' 必须用户指定。[visualMap.min, visualMax.max] 形成了视觉映射的『定义域』。
     */
    private double max = 1D;

    /**
     * 是否显示 visualMap-continuous 组件。如果设置为 false，不会显示，但是数据映射的功能还存在。
     */
    private boolean show = false;

    /**
     * 指定用数据的『哪个维度』，映射到视觉元素上  默认取 data 中最后一个维度。
     */
    private int dimension = 2;
    /**
     * 定义 在选中范围中 的视觉元素。 目前定义不完善
     */
    private InRange inRange = new InRange();

    public JSONObject getJSONObject(){
        JSONObject visualMap = JSONKit.create();
        visualMap.put("show",this.isShow());
        visualMap.put("dimension",this.getDimension());
        visualMap.put("min",this.getMin());
        visualMap.put("max",this.getMax());
        visualMap.put("inRange",this.getInRange().getJSONObject());
        return visualMap;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public double getMin() {
        return min;
    }

    public void setMin(double min) {
        this.min = min;
    }

    public double getMax() {
        return max;
    }

    public void setMax(double max) {
        this.max = max;
    }

    public boolean isShow() {
        return show;
    }

    public void setShow(boolean show) {
        this.show = show;
    }

    public int getDimension() {
        return dimension;
    }

    public void setDimension(int dimension) {
        this.dimension = dimension;
    }

    public InRange getInRange() {
        return inRange;
    }

    public void setInRange(InRange inRange) {
        this.inRange = inRange;
    }

    public static class InRange{
        private List<Color> color;

        public List<Color> getColor() {
            return color;
        }

        public void setColor(List<Color> color) {
            this.color = color;
        }

        public static List<Color> getDefaultColor(){
            List<Color> colors = new ArrayList<>();
            colors.add(new Color(0x313695));
            colors.add(new Color(0x4575b4));
            colors.add(new Color(0x74add1));
            colors.add(new Color(0xabd9e9));
            colors.add(new Color(0xe0f3f8));
            colors.add(new Color(0xffffbf));
            colors.add(new Color(0xfee090));
            colors.add(new Color(0xfdae61));
            colors.add(new Color(0xf46d43));
            colors.add(new Color(0xd73027));
            colors.add(new Color(0xa50026));
            return colors;
        }

        public JSONObject getJSONObject(){
            JSONObject inRange = JSONKit.create();
            JSONArray colorJSONArray = JSONKit.createJSONArray();
            if(this.getColor() == null){
                this.setColor(getDefaultColor());
            }
            for (int i = 0; i < color.size(); i++) {
                Color color = this.color.get(i);
                colorJSONArray.put(PluginStringUtils.colorToString(color));
            }
            inRange.put("color",colorJSONArray);

            return inRange;
        }
    }
}
