package com.fr.plugin.sqy.surface.pane.style;


import com.fr.design.formula.TinyFormulaPane;
import com.fr.design.gui.ilable.UILabel;
import com.fr.design.gui.ispinner.UISpinner;
import com.fr.design.gui.ispinner.chart.UISpinnerWithPx;
import com.fr.design.layout.TableLayoutHelper;
import com.fr.design.style.color.ColorSelectBox;
import com.fr.extended.chart.ExtendedScrollPane;
import com.fr.plugin.chart.type.LineType;
import com.fr.plugin.sqy.surface.chart.Chart;
import com.fr.plugin.sqy.surface.config.LineStyleConfig;
import com.fr.plugin.sqy.surface.option.VisualMap;
import com.fr.plugin.sqy.surface.util.CreatePaneUtils;
import com.fr.plugin.sqy.surface.zenum.LineTypeEnum;
import com.fr.van.chart.designer.TableLayout4VanChartHelper;
import com.fr.van.chart.designer.component.LineTypeComboBox;
import com.fr.van.chart.designer.component.VanChartFillStylePane;

import javax.swing.*;
import java.awt.*;

public class VisualPane extends ExtendedScrollPane<Chart> {
    private VanChartFillStylePane vanChartFillStylePane;
    private TinyFormulaPane visualMin;
    private TinyFormulaPane visualMax;
    private LineTypeComboBox lineTypeComboBox;
    private UISpinner lineWidthSpinner;
    private ColorSelectBox colorButton;




    @Override
    public void populateBean(Chart ob) {
        if(ob.getColorFillStyle().getColorStyle() == 0 || ob.getColorFillStyle().getColorStyle() == 2){
            ob.getColorFillStyle().setColorStyle(1);
            ob.getColorFillStyle().setColorList(VisualMap.InRange.getDefaultColor());
        }
        this.vanChartFillStylePane.populateBean(ob.getColorFillStyle());
        this.visualMin.populateBean(ob.getVisualMin().getContent());
        this.visualMax.populateBean(ob.getVisualMax().getContent());

        LineStyleConfig lineStyleConfig = ob.getWireFrameConfig();
        this.lineTypeComboBox.setSelectedIndex(LineTypeEnum.parseString(lineStyleConfig.getType()).ordinal());
        this.lineWidthSpinner.setValue(lineStyleConfig.getWidth());
        this.colorButton.setSelectObject(lineStyleConfig.getColor());
    }

    @Override
    public void updateBean(Chart ob) {
        if(this.vanChartFillStylePane.updateBean().getColorStyle() == 0 || this.vanChartFillStylePane.updateBean().getColorStyle() == 2){
            ob.getColorFillStyle().setColorStyle(1);
            ob.getColorFillStyle().setColorList(VisualMap.InRange.getDefaultColor());
        }else {
            ob.setColorFillStyle(this.vanChartFillStylePane.updateBean());
        }
        ob.getVisualMin().setContent(this.visualMin.updateBean());
        ob.getVisualMax().setContent(this.visualMax.updateBean());


        ob.getWireFrameConfig().setType(LineTypeEnum.parseInt(this.lineTypeComboBox.getSelectedIndex()).getName());
        ob.getWireFrameConfig().setWidth(this.lineWidthSpinner.getValue());
        ob.getWireFrameConfig().setColor(this.colorButton.getSelectObject());
    }


    private JPanel getColorPane() {
        JPanel var1 = new JPanel(new BorderLayout());
        this.vanChartFillStylePane = new VanChartFillStylePane();
        var1.add(this.vanChartFillStylePane, "Center");

        JPanel var2 = TableLayout4VanChartHelper.createExpandablePaneWithTitle("曲面颜色(仅能使用自定义)", var1);
        var1.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 0));
        return var2;
    }

    private JPanel getAreaPane(){
        this.visualMin = new TinyFormulaPane();
        this.visualMax = new TinyFormulaPane();
        JPanel contentPane = CreatePaneUtils.createGapTableLayoutPane(new Component[][]{{null, null}, {new UILabel("最小值"), this.visualMin}, {new UILabel("最大值"), this.visualMax}});
        JPanel var2 = TableLayout4VanChartHelper.createExpandablePaneWithTitle("视觉范围(Z维度)", contentPane);
        return var2;
    }

    private JPanel getWireframePane(){
        this.lineTypeComboBox = new LineTypeComboBox(new LineType[]{LineType.NONE, LineType.SOLID, LineType.DASHED});
        this.lineWidthSpinner = new UISpinnerWithPx(1D, 2.147483647E9D, 0.5D, 2.0D);
        this.colorButton = new ColorSelectBox(1);

        JPanel contentPane = CreatePaneUtils.createGapTableLayoutPane(new Component[][]{{null, null}
                , {new UILabel("线型"), this.lineTypeComboBox}
                , {new UILabel("线宽"), this.lineWidthSpinner}
                , {new UILabel("颜色"), this.colorButton}});
        JPanel var2 = TableLayout4VanChartHelper.createExpandablePaneWithTitle("曲面间隔线", contentPane);
        return var2;
    }

    @Override
    protected JPanel createContentPane() {
        JPanel panel = new JPanel(new BorderLayout());
        Component[][] components = {{null}
                , {this.getColorPane()}
                , {this.getAreaPane()}
                , {this.getWireframePane()}
        };
        panel.add(TableLayoutHelper.createTableLayoutPane(components,1), BorderLayout.CENTER);
        return panel;
    }

    @Override
    protected String title4PopupWindow() {
        return "视觉";
    }
}
